==> idkit-0.2.5.tar.gz: lib/src/widget/slider/widget_slider_range.dart <==
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:idkit/idkit.dart';
import 'package:idkit/src/widget/slider/widget/slider_floatingmark.dart';
import 'package:idkit/src/widget/slider/widget/slider_thumb.dart';
import 'package:idkit/src/widget/slider/widget/slider_track.dart';

class CustomRangeSlider extends StatefulWidget {
  const CustomRangeSlider({
    super.key,
    this.initLeftValue = 0,
    this.initRightValue = 1,
    this.maxValue = 1,
    this.minValue = 0,
    this.thumbSize = 6,
    this.thumbColor = Colors.blue,
    this.thumbAnimationDuration = const Duration(milliseconds: 200),
    this.trackIndent = 3,
    this.trackHeight = 3,
    this.trackColor = Colors.grey,
    this.trackActiveColor = Colors.blue,
    this.trackActiveGradientColors,
    this.trackActiveGradientStops,
    this.trackActiveGradientBegin = Alignment.centerLeft,
    this.trackActiveGradientEnd = Alignment.centerRight,
    this.trackActiveGradientTileMode = TileMode.clamp,
    this.floatingMark = true,
    this.floatingMarkWidth = 1,
    this.floatingMarkIndent = 3,
    this.floatingMarkColor = Colors.blue,
    this.sliderValueMethod,
  })  : assert(initLeftValue <= initRightValue,
            'The initial value on the left of the range track is not greater than the initial value on the right.'),
        assert(minValue <= maxValue,
            'The minimum value of the range track is not greater than the maximum value.'),
        assert(minValue <= initLeftValue && initLeftValue <= maxValue,
            'The left initial value of the range track should be no less than the track min and no more than the track max.'),
        assert(minValue <= initRightValue && initRightValue <= maxValue,
            'The right initial value of the range track should be no less than the track min and no more than the track max.');

  /// The initial value of the left side of the range track.
  /// The initial value range of [initValue] is 0.0 to 1.0.
  final double initLeftValue;

  /// The initial value of the right side of the range track.
  /// The initial value range of [initValue] is 0.0 to 1.0.
  final double initRightValue;

  /// The [maxValue] that the sliding rail can slide.
  /// The [maxValue] ranges from 0.0 to 1.0, and it is greater than or equal to the [minValue].
  final double maxValue;

  /// The [minValue] that the sliding track can slide.
  /// The [minValue] ranges from 0.0 to 1.0, and it is less than or equal to the [maxValue].
  final double minValue;

  /// The size of the thumb slider.
  final double thumbSize;

  /// The color of the thumb block.
  final Color thumbColor;

  /// Thumb block animation time.
  final Duration thumbAnimationDuration;

  /// Indentation of sliding track.
  final double trackIndent;

  /// Height of sliding track.
  final double trackHeight;

  /// The color of the slide track.
  final Color trackColor;

  /// The active color of the slider track.
  final Color trackActiveColor;

  /// A collection of active gradient colors for the slide track.
  final List<Color>? trackActiveGradientColors;

  /// Active track gradient color ratio.
  final List<double>? trackActiveGradientStops;

  /// Active track fade start orientation.
  final AlignmentGeometry trackActiveGradientBegin;

  /// Active track fade end orientation.
  final AlignmentGeometry trackActiveGradientEnd;

  /// Active track gradient tile mode.
  final TileMode trackActiveGradientTileMode;

  /// Whether to enable the auxiliary hover function. It only works when using Apple Pecile.
  final bool floatingMark;

  /// The width of the floating mark.
  final double floatingMarkWidth;

  /// Indentation of floating mark.
  final double floatingMarkIndent;

  /// The color of the floating mark.
  final Color floatingMarkColor;

  /// Callback function for the current sliding value of the sliding track.
  final Function(double leftValue, double rightValue)? sliderValueMethod;

  @override
  State<CustomRangeSlider> createState() => _CustomRangeSliderState();
}

class _CustomRangeSliderState extends State<CustomRangeSlider>
    with TickerProviderStateMixin {
  /// Swipe track subscribers.
  final StreamController<double> leftSliderStreamController =
      StreamController.broadcast();
  final StreamController<double> rightSliderStreamController =
      StreamController.broadcast();
  final StreamController<TrackRangeInfo> rangeSliderStreamController =
      StreamController.broadcast();

  ///  An animation controller for the thumb block.
  late AnimationController leftThumbAnimationController;
  late AnimationController rightThumbAnimationController;

  /// Subscribers to the floating mark.
  late StreamController<FloatingMarkInfo> floatingMarkStream =
      StreamController<FloatingMarkInfo>.broadcast();

  /// The display mode of the active track.
  late bool isActiveTrackMode = false;

  /// Width of the slide track part.
  late double sliderWWidth = 0;

  /// Slide track initial offset position.
  late double initLocation = 0;

  /// Maximum offset position of sliding track.
  late double maxOffset = 0;

  /// Minimum offset position of sliding track.
  late double minOffset = 0;

  /// The marker of the thumb block that is being manipulated.
  /// true: right  false:left
  late bool isThere = false;

  /// Thumb block offset position.
  late double leftOffset = 0;
  late double rightOffset = 0;

  /// Get the track offset.
  double _getOffset(double value) =>
      widget.trackIndent + value * (sliderWWidth - widget.trackIndent * 2);
  double _getValue(double value) =>
      (value - widget.trackIndent) / (sliderWWidth - widget.trackIndent * 2);

  @override
  void initState() {
    leftThumbAnimationController = AnimationController(
      vsync: this,
      duration: widget.thumbAnimationDuration,
      reverseDuration: widget.thumbAnimationDuration,
      upperBound: 1.5,
      lowerBound: 1,
    );
    rightThumbAnimationController = AnimationController(
      vsync: this,
      duration: widget.thumbAnimationDuration,
      reverseDuration: widget.thumbAnimationDuration,
      upperBound: 1.5,
      lowerBound: 1,
    );
    isActiveTrackMode = widget.trackActiveGradientColors != null &&
        widget.trackActiveGradientColors!.length > 1;
    WidgetsBinding.instance.addPostFrameCallback((_) {
      leftOffset = _getOffset(widget.initLeftValue
          .range(min: widget.minValue, max: widget.maxValue));
      leftSliderStreamController.add(leftOffset);
      rightOffset = _getOffset(widget.initRightValue
          .range(min: widget.minValue, max: widget.maxValue));
      rightSliderStreamController.add(rightOffset);
      rangeSliderStreamController.add(
          TrackRangeInfo(leftLocation: leftOffset, rightLocation: rightOffset));
      maxOffset = _getOffset(widget.maxValue);
      minOffset = _getOffset(widget.minValue);
    });
    super.initState();
  }

  @override
  void didUpdateWidget(covariant CustomRangeSlider oldWidget) {
    if (oldWidget.initLeftValue != widget.initLeftValue) {
      leftOffset = _getOffset(widget.initLeftValue
          .range(min: widget.minValue, max: widget.maxValue));
      leftSliderStreamController.add(leftOffset);
      rangeSliderStreamController.add(
          TrackRangeInfo(leftLocation: leftOffset, rightLocation: rightOffset));
    }

    if (oldWidget.initRightValue != widget.initRightValue) {
      rightOffset = _getOffset(widget.initRightValue
          .range(min: widget.minValue, max: widget.maxValue));
      rightSliderStreamController.add(rightOffset);
      rangeSliderStreamController.add(
          TrackRangeInfo(leftLocation: leftOffset, rightLocation: rightOffset));
    }

    if (oldWidget.trackActiveGradientColors !=
        widget.trackActiveGradientColors) {
      isActiveTrackMode = widget.trackActiveGradientColors != null &&
          widget.trackActiveGradientColors!.length > 1;
    }

    if (oldWidget.maxValue != widget.maxValue) {
      maxOffset = _getOffset(widget.maxValue);
    }

    if (oldWidget.minValue != widget.minValue) {
      minOffset = _getOffset(widget.minValue);
    }

    if (oldWidget.trackIndent != widget.trackIndent) {
      maxOffset = _getOffset(widget.maxValue);
      minOffset = _getOffset(widget.minValue);
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (_, BoxConstraints boxConstraints) {
        sliderWWidth = boxConstraints.biggest.width;
        return MouseRegion(
          onHover: widget.floatingMark
              ? (event) => floatingMarkStream.add(FloatingMarkInfo(
                    state: true,
                    position: event.localPosition.dx,
                  ))
              : null,
          onEnter: widget.floatingMark
              ? (event) => floatingMarkStream.add(FloatingMarkInfo(
                    state: true,
                    position: event.localPosition.dx,
                  ))
              : null,
          onExit: widget.floatingMark
              ? (event) => floatingMarkStream.add(FloatingMarkInfo(
                    state: false,
                    position: event.localPosition.dx,
                  ))
              : null,
          child: GestureDetector(
            behavior: HitTestBehavior.translucent,
            onHorizontalDragDown: (details) {
              final double leftLocation =
                  (details.localPosition.dx - leftOffset).abs();
              final double rightLocation =
                  (details.localPosition.dx - rightOffset).abs();
              if (leftLocation != rightLocation) {
                isThere = leftLocation > rightLocation;
              } else {
                isThere = !(details.localPosition.dx - leftOffset).isNegative;
              }
              if (widget.floatingMark) {
                floatingMarkStream.add(const FloatingMarkInfo(state: false));
              }
              if (isThere) {
                rightThumbAnimationController.forward();
              } else {
                leftThumbAnimationController.forward();
              }
            },
            onHorizontalDragUpdate: (details) {
              final double location = details.localPosition.dx
                  .range(min: minOffset, max: maxOffset);
              if (isThere) {
                rightOffset = location;
                rightSliderStreamController.add(rightOffset);
              } else {
                leftOffset = location;
                leftSliderStreamController.add(leftOffset);
              }
              rangeSliderStreamController.add(TrackRangeInfo(
                  leftLocation: leftOffset, rightLocation: rightOffset));
              widget.sliderValueMethod
                  ?.call(_getValue(leftOffset), _getValue(rightOffset));
            },
            onHorizontalDragEnd: (details) {
              if (isThere) {
                rightThumbAnimationController.reverse();
              } else {
                leftThumbAnimationController.reverse();
              }
            },
            child: Stack(
              children: [
                Positioned.fill(
                  child: Align(
                    alignment: Alignment.center,
                    child: SliderTrack(
                      key: UniqueKey(),
                      height: widget.trackHeight,
                      indent: widget.trackIndent,
                      color: widget.trackColor,
                    ),
                  ),
                ),
                StreamBuilder<TrackRangeInfo>(
                  initialData: TrackRangeInfo(
                      rightLocation: sliderWWidth - widget.trackIndent),
                  stream: rangeSliderStreamController.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data!.leftLocation,
                      child: Align(
                        alignment: Alignment.centerLeft,
                        child: Container(
                          height: widget.trackHeight,
                          width: snapshot.data!.distance,
                          decoration: BoxDecoration(
                            borderRadius:
                                BorderRadius.circular(widget.trackHeight),
                            color: widget.trackActiveColor,
                            gradient: isActiveTrackMode
                                ? LinearGradient(
                                    begin: widget.trackActiveGradientBegin,
                                    end: widget.trackActiveGradientEnd,
                                    colors: widget.trackActiveGradientColors!,
                                    stops: widget.trackActiveGradientStops,
                                    tileMode:
                                        widget.trackActiveGradientTileMode,
                                  )
                                : null,
                          ),
                        ),
                      ),
                    );
                  },
                ),
                StreamBuilder<FloatingMarkInfo>(
                  initialData: const FloatingMarkInfo(),
                  stream: floatingMarkStream.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data!.position -
                          widget.floatingMarkWidth * 0.5,
                      child: Visibility(
                        visible: snapshot.data!.state,
                        child: Align(
                          alignment: Alignment.centerLeft,
                          child: FloatingMark(
                            width: widget.floatingMarkWidth,
                            color: widget.floatingMarkColor,
                            indent: widget.floatingMarkIndent,
                          ),
                        ),
                      ),
                    );
                  },
                ),
                StreamBuilder<double>(
                  initialData: widget.trackIndent,
                  stream: leftSliderStreamController.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data! - widget.thumbSize * 0.5,
                      child: OverflowBox(
                        child: Align(
                          alignment: Alignment.centerLeft,
                          child: AnimatedBuilder(
                            animation: leftThumbAnimationController,
                            builder: (context, child) {
                              return Transform.scale(
                                scale: leftThumbAnimationController.value,
                                child: SliderThumb(
                                  size: widget.thumbSize,
                                  color: widget.thumbColor,
                                ),
                              );
                            },
                          ),
                        ),
                      ),
                    );
                  },
                ),
                StreamBuilder<double>(
                  initialData: sliderWWidth - widget.trackIndent,
                  stream: rightSliderStreamController.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data! - widget.thumbSize * 0.5,
                      child: Align(
                        alignment: Alignment.centerLeft,
                        child: AnimatedBuilder(
                          animation: rightThumbAnimationController,
                          builder: (context, child) {
                            return Transform.scale(
                              scale: rightThumbAnimationController.value,
                              child: SliderThumb(
                                size: widget.thumbSize,
                                color: widget.thumbColor,
                              ),
                            );
                          },
                        ),
                      ),
                    );
                  },
                )
              ],
            ),
          ),
        );
      },
    );
  }

  @override
  void dispose() {
    leftSliderStreamController.close();
    rightSliderStreamController.close();
    rangeSliderStreamController.close();
    floatingMarkStream.close();
    leftThumbAnimationController.dispose();
    rightThumbAnimationController.dispose();
    super.dispose();
  }
}

==> idkit-0.3.2.tar.gz: lib/src/widget/slider/widget_slider_range.dart <==
import 'dart:async';
import 'package:flutter/material.dart';
import 'package:idkit/idkit.dart';
import 'package:idkit/src/widget/slider/widget/slider_floatingmark.dart';
import 'package:idkit/src/widget/slider/widget/slider_thumb.dart';
import 'package:idkit/src/widget/slider/widget/slider_track.dart';

class CustomRangeSlider extends StatefulWidget {
  const CustomRangeSlider({
    super.key,
    this.initLeftValue = 0,
    this.initRightValue = 1,
    this.maxValue = 1,
    this.minValue = 0,
    this.thumbSize = 6,
    this.thumbColor = Colors.blue,
    this.thumbAnimationDuration = const Duration(milliseconds: 200),
    this.trackIndent = 3,
    this.trackHeight = 3,
    this.trackColor = Colors.grey,
    this.trackActiveColor = Colors.blue,
    this.trackActiveGradientColors,
    this.trackActiveGradientStops,
    this.trackActiveGradientBegin = Alignment.centerLeft,
    this.trackActiveGradientEnd = Alignment.centerRight,
    this.trackActiveGradientTileMode = TileMode.clamp,
    this.floatingMark = true,
    this.floatingMarkWidth = 1,
    this.floatingMarkIndent = 3,
    this.floatingMarkColor = Colors.blue,
    this.sliderValueMethod,
  })  : assert(initLeftValue <= initRightValue,
            'The initial value on the left of the range track is not greater than the initial value on the right.'),
        assert(minValue <= maxValue,
            'The minimum value of the range track is not greater than the maximum value.'),
        assert(minValue <= initLeftValue && initLeftValue <= maxValue,
            'The left initial value of the range track should be no less than the track min and no more than the track max.'),
        assert(minValue <= initRightValue && initRightValue <= maxValue,
            'The right initial value of the range track should be no less than the track min and no more than the track max.');

  /// The initial value of the left side of the range track.
  /// The initial value range of [initValue] is 0.0 to 1.0.
  final double initLeftValue;

  /// The initial value of the right side of the range track.
  /// The initial value range of [initValue] is 0.0 to 1.0.
  final double initRightValue;

  /// The [maxValue] that the sliding rail can slide.
  /// The [maxValue] ranges from 0.0 to 1.0, and it is greater than or equal to the [minValue].
  final double maxValue;

  /// The [minValue] that the sliding track can slide.
  /// The [minValue] ranges from 0.0 to 1.0, and it is less than or equal to the [maxValue].
  final double minValue;

  /// The size of the thumb slider.
  final double thumbSize;

  /// The color of the thumb block.
  final Color thumbColor;

  /// Thumb block animation time.
  final Duration thumbAnimationDuration;

  /// Indentation of sliding track.
  final double trackIndent;

  /// Height of sliding track.
  final double trackHeight;

  /// The color of the slide track.
  final Color trackColor;

  /// The active color of the slider track.
  final Color trackActiveColor;

  /// A collection of active gradient colors for the slide track.
  final List<Color>? trackActiveGradientColors;

  /// Active track gradient color ratio.
  final List<double>? trackActiveGradientStops;

  /// Active track fade start orientation.
  final AlignmentGeometry trackActiveGradientBegin;

  /// Active track fade end orientation.
  final AlignmentGeometry trackActiveGradientEnd;

  /// Active track gradient tile mode.
  final TileMode trackActiveGradientTileMode;

  /// Whether to enable the auxiliary hover function. It only works when using Apple Pecile.
  final bool floatingMark;

  /// The width of the floating mark.
  final double floatingMarkWidth;

  /// Indentation of floating mark.
  final double floatingMarkIndent;

  /// The color of the floating mark.
  final Color floatingMarkColor;

  /// Callback function for the current sliding value of the sliding track.
  final Function(double leftValue, double rightValue)? sliderValueMethod;

  @override
  State<CustomRangeSlider> createState() => _CustomRangeSliderState();
}

class _CustomRangeSliderState extends State<CustomRangeSlider>
    with TickerProviderStateMixin {
  /// Swipe track subscribers.
  final StreamController<double> leftSliderStreamController =
      StreamController.broadcast();
  final StreamController<double> rightSliderStreamController =
      StreamController.broadcast();
  final StreamController<TrackRangeInfo> rangeSliderStreamController =
      StreamController.broadcast();

  ///  An animation controller for the thumb block.
  late AnimationController leftThumbAnimationController;
  late AnimationController rightThumbAnimationController;

  /// Subscribers to the floating mark.
  late StreamController<FloatingMarkInfo> floatingMarkStream =
      StreamController<FloatingMarkInfo>.broadcast();

  /// The display mode of the active track.
  late bool isActiveTrackMode = false;

  /// Width of the slide track part.
  late double sliderWWidth = 0;

  /// Slide track initial offset position.
  late double initLocation = 0;

  /// Maximum offset position of sliding track.
  late double maxOffset = 0;

  /// Minimum offset position of sliding track.
  late double minOffset = 0;

  /// The marker of the thumb block that is being manipulated.
  /// true: right  false:left
  late bool isThere = false;

  /// Thumb block offset position.
  late double leftOffset = 0;
  late double rightOffset = 0;

  /// Get the track offset.
  double _getOffset(double value) =>
      widget.trackIndent + value * (sliderWWidth - widget.trackIndent * 2);
  double _getValue(double value) =>
      (value - widget.trackIndent) / (sliderWWidth - widget.trackIndent * 2);

  @override
  void initState() {
    leftThumbAnimationController = AnimationController(
      vsync: this,
      duration: widget.thumbAnimationDuration,
      reverseDuration: widget.thumbAnimationDuration,
      upperBound: 1.5,
      lowerBound: 1,
    );
    rightThumbAnimationController = AnimationController(
      vsync: this,
      duration: widget.thumbAnimationDuration,
      reverseDuration: widget.thumbAnimationDuration,
      upperBound: 1.5,
      lowerBound: 1,
    );
    isActiveTrackMode = widget.trackActiveGradientColors != null &&
        widget.trackActiveGradientColors!.length > 1;
    WidgetsBinding.instance.addPostFrameCallback((_) {
      leftOffset = _getOffset(widget.initLeftValue
          .range(min: widget.minValue, max: widget.maxValue));
      leftSliderStreamController.add(leftOffset);
      rightOffset = _getOffset(widget.initRightValue
          .range(min: widget.minValue, max: widget.maxValue));
      rightSliderStreamController.add(rightOffset);
      rangeSliderStreamController.add(
          TrackRangeInfo(leftLocation: leftOffset, rightLocation: rightOffset));
      maxOffset = _getOffset(widget.maxValue);
      minOffset = _getOffset(widget.minValue);
    });
    super.initState();
  }

  @override
  void didUpdateWidget(covariant CustomRangeSlider oldWidget) {
    if (oldWidget.initLeftValue != widget.initLeftValue) {
      leftOffset = _getOffset(widget.initLeftValue
          .range(min: widget.minValue, max: widget.maxValue));
      leftSliderStreamController.add(leftOffset);
      rangeSliderStreamController.add(
          TrackRangeInfo(leftLocation: leftOffset, rightLocation: rightOffset));
    }

    if (oldWidget.initRightValue != widget.initRightValue) {
      rightOffset = _getOffset(widget.initRightValue
          .range(min: widget.minValue, max: widget.maxValue));
      rightSliderStreamController.add(rightOffset);
      rangeSliderStreamController.add(
          TrackRangeInfo(leftLocation: leftOffset, rightLocation: rightOffset));
    }

    if (oldWidget.trackActiveGradientColors !=
        widget.trackActiveGradientColors) {
      isActiveTrackMode = widget.trackActiveGradientColors != null &&
          widget.trackActiveGradientColors!.length > 1;
    }

    if (oldWidget.maxValue != widget.maxValue) {
      maxOffset = _getOffset(widget.maxValue);
    }

    if (oldWidget.minValue != widget.minValue) {
      minOffset = _getOffset(widget.minValue);
    }

    if (oldWidget.trackIndent != widget.trackIndent) {
      maxOffset = _getOffset(widget.maxValue);
      minOffset = _getOffset(widget.minValue);
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) {
    return LayoutBuilder(
      builder: (_, BoxConstraints boxConstraints) {
        sliderWWidth = boxConstraints.biggest.width;
        return MouseRegion(
          onHover: widget.floatingMark
              ? (event) => floatingMarkStream.add(FloatingMarkInfo(
                    state: true,
                    position: event.localPosition.dx
                        .range(min: minOffset, max: maxOffset),
                  ))
              : null,
          onEnter: widget.floatingMark
              ? (event) => floatingMarkStream.add(FloatingMarkInfo(
                    state: true,
                    position: event.localPosition.dx
                        .range(min: minOffset, max: maxOffset),
                  ))
              : null,
          onExit: widget.floatingMark
              ? (event) => floatingMarkStream.add(FloatingMarkInfo(
                    state: false,
                    position: event.localPosition.dx
                        .range(min: minOffset, max: maxOffset),
                  ))
              : null,
          child: GestureDetector(
            behavior: HitTestBehavior.translucent,
            onHorizontalDragDown: (details) {
              final double leftLocation =
                  (details.localPosition.dx - leftOffset).abs();
              final double rightLocation =
                  (details.localPosition.dx - rightOffset).abs();
              if (leftLocation != rightLocation) {
                isThere = leftLocation > rightLocation;
              } else {
                isThere = !(details.localPosition.dx - leftOffset).isNegative;
              }
              if (widget.floatingMark) {
                floatingMarkStream.add(const FloatingMarkInfo(state: false));
              }
              if (isThere) {
                rightThumbAnimationController.forward();
              } else {
                leftThumbAnimationController.forward();
              }
            },
            onHorizontalDragUpdate: (details) {
              final double location = details.localPosition.dx
                  .range(min: minOffset, max: maxOffset);
              if (isThere) {
                rightOffset = location;
                rightSliderStreamController.add(rightOffset);
              } else {
                leftOffset = location;
                leftSliderStreamController.add(leftOffset);
              }
              rangeSliderStreamController.add(TrackRangeInfo(
                  leftLocation: leftOffset, rightLocation: rightOffset));
              widget.sliderValueMethod
                  ?.call(_getValue(leftOffset), _getValue(rightOffset));
            },
            onHorizontalDragEnd: (details) {
              if (isThere) {
                rightThumbAnimationController.reverse();
              } else {
                leftThumbAnimationController.reverse();
              }
            },
            child: Stack(
              children: [
                Positioned.fill(
                  child: Align(
                    alignment: Alignment.center,
                    child: SliderTrack(
                      key: UniqueKey(),
                      height: widget.trackHeight,
                      indent: widget.trackIndent,
                      color: widget.trackColor,
                    ),
                  ),
                ),
                StreamBuilder<TrackRangeInfo>(
                  initialData: TrackRangeInfo(
                      rightLocation: sliderWWidth - widget.trackIndent),
                  stream: rangeSliderStreamController.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data!.leftLocation,
                      child: Align(
                        alignment: Alignment.centerLeft,
                        child: Container(
                          height: widget.trackHeight,
                          width: snapshot.data!.distance,
                          decoration: BoxDecoration(
                            borderRadius:
                                BorderRadius.circular(widget.trackHeight),
                            color: widget.trackActiveColor,
                            gradient: isActiveTrackMode
                                ? LinearGradient(
                                    begin: widget.trackActiveGradientBegin,
                                    end: widget.trackActiveGradientEnd,
                                    colors: widget.trackActiveGradientColors!,
                                    stops: widget.trackActiveGradientStops,
                                    tileMode:
                                        widget.trackActiveGradientTileMode,
                                  )
                                : null,
                          ),
                        ),
                      ),
                    );
                  },
                ),
                StreamBuilder<FloatingMarkInfo>(
                  initialData: const FloatingMarkInfo(),
                  stream: floatingMarkStream.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data!.position -
                          widget.floatingMarkWidth * 0.5,
                      child: Visibility(
                        visible: snapshot.data!.state,
                        child: Align(
                          alignment: Alignment.centerLeft,
                          child: FloatingMark(
                            width: widget.floatingMarkWidth,
                            color: widget.floatingMarkColor,
                            indent: widget.floatingMarkIndent,
                          ),
                        ),
                      ),
                    );
                  },
                ),
                StreamBuilder<double>(
                  initialData: widget.trackIndent,
                  stream: leftSliderStreamController.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data! - widget.thumbSize * 0.5,
                      child: OverflowBox(
                        child: Align(
                          alignment: Alignment.centerLeft,
                          child: AnimatedBuilder(
                            animation: leftThumbAnimationController,
                            builder: (context, child) {
                              return Transform.scale(
                                scale: leftThumbAnimationController.value,
                                child: SliderThumb(
                                  size: widget.thumbSize,
                                  color: widget.thumbColor,
                                ),
                              );
                            },
                          ),
                        ),
                      ),
                    );
                  },
                ),
                StreamBuilder<double>(
                  initialData: sliderWWidth - widget.trackIndent,
                  stream: rightSliderStreamController.stream,
                  builder: (context, snapshot) {
                    return Positioned.fill(
                      left: snapshot.data! - widget.thumbSize * 0.5,
                      child: Align(
                        alignment: Alignment.centerLeft,
                        child: AnimatedBuilder(
                          animation: rightThumbAnimationController,
                          builder: (context, child) {
                            return Transform.scale(
                              scale: rightThumbAnimationController.value,
                              child: SliderThumb(
                                size: widget.thumbSize,
                                color: widget.thumbColor,
                              ),
                            );
                          },
                        ),
                      ),
                    );
                  },
                )
              ],
            ),
          ),
        );
      },
    );
  }

  @override
  void dispose() {
    leftSliderStreamController.close();
    rightSliderStreamController.close();
    rangeSliderStreamController.close();
    floatingMarkStream.close();
    leftThumbAnimationController.dispose();
    rightThumbAnimationController.dispose();
    super.dispose();
  }
}
